@@ -31,6 +31,7 @@
 #include <QPushButton>
 #include <QTimer>
 #include <QCoreApplication>
+#include <QMessageBox>
 #if QT_VERSION >= 0x050000
 #include <QStandardPaths>
 #else
@@ -39,12 +40,14 @@
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
-    m_topToolBar(new QToolBar(this)),
+    m_actionToolBar(new QToolBar(this)),
+    m_filterToolBar(new QToolBar(this)),
     m_bottomToolBar(new QToolBar(this)),
     m_startAction(new QAction(QIcon::fromTheme("media-playback-start"), tr("Start"), this)),
     m_pauseAction(new QAction(QIcon::fromTheme("media-playback-pause"), tr("Pause"), this)),
     m_searchEdit(new QLineEdit(this)),
     m_filterComboBox(new QComboBox(this)),
+    m_nextActionComboBox(new QComboBox(this)),
     m_optionsButton(new QToolButton(this)),
     m_optionsMenu(new QMenu(m_optionsButton)),
     m_concurrentMenu(m_optionsMenu->addMenu(tr("Maximum concurrent downloads"))),
@@ -95,6 +98,7 @@ MainWindow::MainWindow(QWidget *parent) :
     m_aboutAction(m_helpMenu->addAction(QIcon::fromTheme("help-about"), tr("About"), this, SLOT(showAboutDialog()))),
     m_checkDialog(new CheckUrlsDialog(this)),
     m_progressDialog(new QProgressDialog(this)),
+    m_progressDialogHasCancelButton(false),
     m_trayIcon(new QSystemTrayIcon(QIcon::fromTheme("qdl"), this)),
     m_trayMenu(new QMenu(this)),
     m_trayAddUrlsAction(new QAction(QIcon::fromTheme("list-add"), tr("Add URLs"), this)),
@@ -155,19 +159,34 @@ MainWindow::MainWindow(QWidget *parent) :
         m_rateLimitGroup->addAction(action);
     }
 
-    m_topToolBar->setAllowedAreas(Qt::TopToolBarArea);
-    m_topToolBar->setMovable(false);
-    m_topToolBar->addAction(m_addUrlsAction);
-    m_topToolBar->addAction(m_importUrlsAction);
-    m_topToolBar->addAction(m_retrieveUrlsAction);
-    m_topToolBar->addSeparator();
-    m_topToolBar->addAction(m_startAction);
-    m_topToolBar->addAction(m_pauseAction);
-    m_topToolBar->addSeparator();
-    m_topToolBar->addWidget(new QLabel(tr("Show") + ":", this));
-    m_topToolBar->addWidget(m_filterComboBox);
-    m_topToolBar->addWidget(m_searchEdit);
-    this->addToolBar(m_topToolBar);
+    m_searchEdit->setPlaceholderText(tr("Search"));
+
+    m_filterComboBox->setModel(new StatusFilterModel(m_filterComboBox));
+
+    m_nextActionComboBox->setModel(new TransferActionModel(m_nextActionComboBox));
+    m_nextActionComboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
+
+    m_actionToolBar->setAllowedAreas(Qt::TopToolBarArea);
+    m_actionToolBar->setMovable(false);
+    m_actionToolBar->addAction(m_addUrlsAction);
+    m_actionToolBar->addAction(m_importUrlsAction);
+    m_actionToolBar->addAction(m_retrieveUrlsAction);
+    m_actionToolBar->addSeparator();
+    m_actionToolBar->addAction(m_startAction);
+    m_actionToolBar->addAction(m_pauseAction);
+    m_actionToolBar->addSeparator();
+    m_actionToolBar->addWidget(new QLabel(tr("After current download(s)") + ":", this));
+    m_actionToolBar->addWidget(m_nextActionComboBox);
+
+    m_filterToolBar->setAllowedAreas(Qt::TopToolBarArea);
+    m_filterToolBar->setMovable(false);
+    m_filterToolBar->addWidget(new QLabel(tr("Show") + ":", this));
+    m_filterToolBar->addWidget(m_filterComboBox);
+    m_filterToolBar->addWidget(m_searchEdit);
+
+    this->addToolBar(m_actionToolBar);
+    this->addToolBarBreak();
+    this->addToolBar(m_filterToolBar);
 
     m_optionsButton->setIcon(QIcon::fromTheme("document-properties"));
     m_optionsButton->setToolTip(tr("Options"));
@@ -237,10 +256,6 @@ MainWindow::MainWindow(QWidget *parent) :
     m_packageLowPriorityAction->setCheckable(true);
     m_packageLowPriorityAction->setActionGroup(m_packagePriorityGroup);
 
-    m_searchEdit->setPlaceholderText(tr("Search"));
-
-    m_filterComboBox->setModel(new StatusFilterModel(m_filterComboBox));
-
     m_view->setModel(m_model);
     m_view->setAlternatingRowColors(true);
     m_view->setHeaderHidden(true);
@@ -280,6 +295,7 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(m_pauseAction, SIGNAL(triggered()), m_model, SLOT(pause()));
     this->connect(m_searchEdit, SIGNAL(textChanged(QString)), m_model, SLOT(setSearchQuery(QString)));
     this->connect(m_filterComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onFilterBoxIndexChanged(int)));
+    this->connect(m_nextActionComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onNextActionIndexChanged(int)));
     this->connect(m_view, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
     this->connect(m_transferMenu, SIGNAL(aboutToShow()), this, SLOT(setTransferMenuActions()));
     this->connect(m_packageMenu, SIGNAL(aboutToShow()), this, SLOT(setPackageMenuActions()));
@@ -301,6 +317,7 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
     this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
     this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
+    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
 
     this->onWindowStateChanged();
     this->onPackageCountChanged(m_model->rowCount());
@@ -383,11 +400,16 @@ void MainWindow::onFilterBoxIndexChanged(int index) {
     m_model->setStatusFilter(static_cast<Transfers::Status>(m_filterComboBox->itemData(index, Qt::UserRole + 1).toInt()));
 }
 
+void MainWindow::onNextActionIndexChanged(int index) {
+    m_model->setNextAction(static_cast<Transfers::Action>(m_nextActionComboBox->itemData(index, Qt::UserRole + 1).toInt()));
+}
+
 void MainWindow::onPackageCountChanged(int count) {
     m_transferMenu->setEnabled(count > 0);
     m_packageMenu->setEnabled(count > 0);
     m_startAction->setEnabled(count > 0);
     m_pauseAction->setEnabled(count > 0);
+    m_nextActionComboBox->setEnabled(count > 0);
     m_filterComboBox->setEnabled(count > 0);
     m_searchEdit->setEnabled(count > 0);
 }
@@ -715,17 +737,15 @@ void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &file
 }
 
 void MainWindow::onUrlRetrieverFinished() {
-    QString text;
+    QString results = UrlRetriever::instance()->resultsString();
 
-    foreach (QUrl url, UrlRetriever::instance()->results()) {
-        text.append(url.toString() + "\n");
+    if (results.isEmpty()) {
+        QMessageBox::information(this, tr("Retrieve URLs"), tr("No supported URLs found"));
     }
-
-    if (!text.isEmpty()) {
-        this->showAddUrlsDialog(text);
+    else {
+        this->showAddUrlsDialog(results);
+        UrlRetriever::instance()->clearResults();
     }
-
-    UrlRetriever::instance()->clearResults();
 }
 
 void MainWindow::showTextFileDialog() {
@@ -754,10 +774,14 @@ void MainWindow::showProgressDialog(const QString &message, int maximum) {
     m_progressDialog->setRange(0, maximum);
 
     if ((this->sender()) && (this->connect(m_progressDialog, SIGNAL(canceled()), this->sender(), SLOT(cancel())))) {
-        m_progressDialog->setCancelButton(new QPushButton(QIcon::fromTheme("process-stop"), tr("Cancel"), m_progressDialog));
+        if (!m_progressDialogHasCancelButton) {
+            m_progressDialog->setCancelButton(new QPushButton(QIcon::fromTheme("process-stop"), tr("Cancel"), m_progressDialog));
+            m_progressDialogHasCancelButton = true;
+        }
     }
     else {
         m_progressDialog->setCancelButton(0);
+        m_progressDialogHasCancelButton = false;
     }
 
     m_progressDialog->open();
