@@ -11,7 +11,11 @@
 #include "../shared/serviceaccountsmodel.h"
 #include "../shared/decaptchaaccountsmodel.h"
 #include "../shared/pluginsettingsmodel.h"
+#include "../shared/packagetransfermodel.h"
+#include "../shared/archivepasswordsmodel.h"
 #include "../shared/definitions.h"
+#include "../dbus/dbusservice.h"
+#include "../dbus/dbusserviceadaptor.h"
 #include "folderlistmodel.h"
 #include "qmlutils.h"
 #include "maskeditem.h"
@@ -24,16 +28,24 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 {
     QScopedPointer<QApplication> app(MDeclarativeCache::qApplication(argc, argv));
     QScopedPointer<QDeclarativeView> view(MDeclarativeCache::qDeclarativeView());
+
     Session session;
     QmlUtils utils;
+    DBusServiceAdaptor adaptor(DBusService::instance());
+
+    QDBusConnection connection = QDBusConnection::sessionBus();
+    connection.registerService("com.marxoft.QDL");
+    connection.registerObject("/", DBusService::instance());
 
-    qRegisterMetaType<ScreenOrientation::Orientation>("ScreenOrientation::Orientation");
-    qRegisterMetaType<Transfers::Status>("Transfers::Status");
-    qRegisterMetaType<Transfers::Priority>("Transfers::Priority");
     qRegisterMetaType< QList<Transfer*> >("QList<Transfer*>");
+    qRegisterMetaType< QList<QUrl> >("QList<QUrl>");
+    qRegisterMetaType<Transfers::Status>("Transfers::Status");
+    qRegisterMetaType<Transfers::Action>("Transfers::Action");
 
     qmlRegisterType<QDeclarativeFolderListModel>("com.marxoft.models",1,0,"FolderListModel");
     qmlRegisterType<ScreenOrientationModel>("com.marxoft.models",1,0,"ScreenOrientationModel");
+    qmlRegisterType<StatusFilterModel>("com.marxoft.models",1,0,"StatusFilterModel");
+    qmlRegisterType<TransferActionModel>("com.marxoft.models",1,0,"TransferActionModel");
     qmlRegisterType<ConcurrentTransfersModel>("com.marxoft.models",1,0,"ConcurrentTransfersModel");
     qmlRegisterType<ConnectionsModel>("com.marxoft.models",1,0,"ConnectionsModel");
     qmlRegisterType<TransferPriorityModel>("com.marxoft.models",1,0,"TransferPriorityModel");
@@ -42,10 +54,16 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<DecaptchaAccountsModel>("com.marxoft.models",1,0,"DecaptchaAccountsModel");
     qmlRegisterType<PluginSettingsModel>("com.marxoft.models",1,0,"PluginSettingsModel");
     qmlRegisterType<NetworkProxyTypeModel>("com.marxoft.models",1,0,"NetworkProxyTypeModel");
+    qmlRegisterType<PackageTransferModel>("com.marxoft.models",1,0,"PackageTransferModel");
+    qmlRegisterType<ArchivePasswordsModel>("com.marxoft.models",1,0,"ArchivePasswordsModel");
     qmlRegisterType<SelectionModel>("com.marxoft.models",1,0,"SelectionModel");
     qmlRegisterType<MaskedItem>("com.marxoft.items",1,0,"MaskedItem");
     qmlRegisterType<Transfer>("com.marxoft.items",1,0,"Transfer");
 
+    qmlRegisterUncreatableType<ScreenOrientation>("com.marxoft.enums",1,0,"ScreenOrientation", "");
+    qmlRegisterUncreatableType<Transfers>("com.marxoft.enums",1,0,"Transfers", "");
+    qmlRegisterUncreatableType<NetworkProxyType>("com.marxoft.enums",1,0,"NetworkProxyType", "");
+
     QDeclarativeContext *context = view->rootContext();
     context->setContextProperty("TransferModel", TransferModel::instance());
     context->setContextProperty("UrlChecker", UrlChecker::instance());
@@ -56,6 +74,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     context->setContextProperty("Database", Database::instance());
     context->setContextProperty("Settings", Settings::instance());
     context->setContextProperty("Utils", &utils);
+    context->setContextProperty("versionNumber", VERSION_NUMBER);
     context->setContextProperty("iconPath", ICON_PATH);
 
     view->setViewport(new QGLWidget());
