@@ -5,9 +5,6 @@
 
 class ServiceAccountsModel;
 class QTreeView;
-class QPushButton;
-class QLineEdit;
-class QModelIndex;
 class QMenu;
 
 class AccountSettingsTab : public QWidget
@@ -20,10 +17,8 @@ public:
 
 private slots:
     void showContextMenu(const QPoint &pos);
-    void addAccount();
     void removeAccount();
     void editAccount();
-    void onAccountEditChanged();
 
 private:
     ServiceAccountsModel *m_model;
@@ -31,9 +26,6 @@ private:
     QMenu *m_contextMenu;
     QAction *m_editAction;
     QAction *m_removeAction;
-    QLineEdit *m_nameEdit;
-    QLineEdit *m_passEdit;
-    QPushButton *m_doneButton;
 };
 
 #endif // ACCOUNTSETTINGSTAB_H
