@@ -79,7 +79,7 @@ MainWindow::MainWindow(QWidget *parent) :
     m_pausedFilterAction(m_transferStatusFilterMenu->addAction(tr("Paused"), this, SLOT(setTransferStatusFilter()))),
     m_failedFilterAction(m_transferStatusFilterMenu->addAction(tr("Failed"), this, SLOT(setTransferStatusFilter()))),
     m_transferStatusFilterGroup(new QActionGroup(this)),
-    m_nextActionMenu(this->menuBar()->addMenu(tr("After current download(s)"))),
+    m_nextActionMenu(m_optionsMenu->addMenu(tr("After current download(s)"))),
     m_continueNextAction(m_nextActionMenu->addAction(tr("Continue"), this, SLOT(setNextAction()))),
     m_pauseNextAction(m_nextActionMenu->addAction(tr("Pause"), this, SLOT(setNextAction()))),
     m_quitNextAction(m_nextActionMenu->addAction(tr("Quit"), this, SLOT(setNextAction()))),
@@ -90,6 +90,8 @@ MainWindow::MainWindow(QWidget *parent) :
     m_connectionsGroup(new QActionGroup(this)),
     m_rateLimitMenu(m_optionsMenu->addMenu(tr("Maximum download speed"))),
     m_rateLimitGroup(new QActionGroup(this)),
+    m_viewMenu(this->menuBar()->addMenu(tr("View"))),
+    m_fullscreenAction(m_viewMenu->addAction(tr("Full screen"), this, SLOT(setFullScreen(bool)))),
     m_preferencesAction(this->menuBar()->addAction(tr("Settings"), this, SLOT(showSettingsDialog()))),
     m_aboutAction(this->menuBar()->addAction(tr("About"), this, SLOT(showAboutDialog()))),
     m_quitAction(this->menuBar()->addAction(tr("Close"), QCoreApplication::instance(), SLOT(quit()))),
@@ -218,8 +220,10 @@ MainWindow::MainWindow(QWidget *parent) :
     m_packageLowPriorityAction->setCheckable(true);
     m_packageLowPriorityAction->setActionGroup(m_packagePriorityGroup);
 
-    m_preferencesAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_S));
-    m_quitAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
+    m_fullscreenAction->setCheckable(true);
+
+    m_preferencesAction->setShortcut(Qt::CTRL + Qt::Key_S);
+    m_quitAction->setShortcut(Qt::CTRL + Qt::Key_Q);
 
     m_view->setModel(m_model);
     m_view->setHeaderHidden(true);
@@ -262,6 +266,52 @@ MainWindow::MainWindow(QWidget *parent) :
 
 MainWindow::~MainWindow() {}
 
+bool MainWindow::event(QEvent *event) {
+    switch (event->type()) {
+    case QEvent::WindowStateChange:
+        m_fullscreenAction->setChecked(this->isFullScreen());
+        break;
+    case QEvent::ContextMenu:
+        return false;
+    default:
+        break;
+    }
+
+    return QMainWindow::event(event);
+}
+
+void MainWindow::keyPressEvent(QKeyEvent *event) {
+    if (!event->isAutoRepeat()) {
+        if (event->modifiers() & Qt::ControlModifier) {
+            switch (event->key()) {
+            case Qt::Key_S:
+                m_preferencesAction->trigger();
+                event->accept();
+                return;
+            case Qt::Key_Q:
+                m_quitAction->trigger();
+                event->accept();
+                return;
+            default:
+                QMainWindow::keyPressEvent(event);
+                return;
+            }
+        }
+
+        switch (event->key()) {
+        case Qt::Key_Up:
+        case Qt::Key_Down:
+            m_view->setFocus(Qt::OtherFocusReason);
+            event->accept();
+            return;
+        default:
+            break;
+        }
+    }
+
+    QMainWindow::keyPressEvent(event);
+}
+
 void MainWindow::showEvent(QShowEvent *event) {
     if (PluginManager::instance()->servicePlugins().isEmpty()) {
         QTimer::singleShot(1000, PluginManager::instance(), SLOT(loadPlugins()));
@@ -270,6 +320,15 @@ void MainWindow::showEvent(QShowEvent *event) {
     QMainWindow::showEvent(event);
 }
 
+void MainWindow::setFullScreen(bool fullScreen) {
+    if (fullScreen) {
+        this->showFullScreen();
+    }
+    else {
+        this->showNormal();
+    }
+}
+
 void MainWindow::onPluginsReady() {
     this->disconnect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(onPluginsReady()));
     m_model->restoreStoredTransfers();
