@@ -23,9 +23,12 @@ public:
     ~MainWindow();
 
 protected:
+    bool event(QEvent *event);
+    void keyPressEvent(QKeyEvent *event);
     void showEvent(QShowEvent *event);
 
 private slots:
+    void setFullScreen(bool fullScreen);
     void onPluginsReady();
     void onPackageCountChanged(int count);
     void setNextAction();
@@ -117,6 +120,8 @@ private:
     QActionGroup *m_connectionsGroup;
     QMenu *m_rateLimitMenu;
     QActionGroup *m_rateLimitGroup;
+    QMenu *m_viewMenu;
+    QAction *m_fullscreenAction;
     QAction *m_preferencesAction;
     QAction *m_aboutAction;
     QAction *m_quitAction;
