@@ -3,8 +3,8 @@
 
 #include <QDialog>
 
+class ArchivePasswordsModel;
 class QListView;
-class QStringListModel;
 class QLineEdit;
 class QPushButton;
 class QMenu;
@@ -22,10 +22,9 @@ private slots:
     void onPasswordEditTextChanged(const QString &text);
     void addArchivePassword();
     void removeArchivePassword();
-    void onArchivePasswordsChanged();
     
 private:
-    QStringListModel *m_model;
+    ArchivePasswordsModel *m_model;
     QListView *m_view;
     QMenu *m_contextMenu;
     QAction *m_removePasswordAction;
