@@ -714,7 +714,7 @@ void Transfer::loadConnections() {
         info.path = this->downloadPath();
         info.name = this->fileName();
         info.size = this->size();
-        info.bytesRemaining = this->size() - this->resumePosition();
+        info.bytesRemaining = this->size() - this->position();
 
         m_file.setMetaInfo(info);
         m_file.start(QThread::LowestPriority);
@@ -1240,13 +1240,7 @@ void Transfer::onCaptchaResponseReady(const QString &response) {
 void Transfer::onDownloadRequestReady(const QNetworkRequest &request, const QByteArray &data) {
     m_request = request;
     m_data = data;
-
-    if (m_connections.isEmpty()) {
-        this->loadConnections();
-    }
-    else {
-        this->addConnections(this->preferredConnections());
-    }
+    this->loadConnections();
 }
 
 void Transfer::onMetaInfoReady(MetaInfo info) {
@@ -1277,6 +1271,7 @@ void Transfer::onBytesDownloaded(qint64 bytes) {
     m_downloadedBytes += bytes;
 #ifdef QML_USER_INTERFACE
     emit positionChanged();
+    emit speedChanged();
 #else
     emit dataChanged(PositionRole);
 #endif
