@@ -4,6 +4,7 @@
 #include "storage.h"
 #include "settings.h"
 #include "urlchecker.h"
+#include "utils.h"
 #include <QDateTime>
 #include <QTimer>
 #include <QCoreApplication>
@@ -12,9 +13,10 @@ TransferModel* TransferModel::m_instance = 0;
 
 TransferModel::TransferModel() :
     QAbstractItemModel(),
-    m_rootItem(new Transfer),
+    m_rootItem(new Transfer(this)),
     m_queueTimer(new QTimer(this)),
-    m_statusFilter(Transfers::Unknown)
+    m_statusFilter(Transfers::Unknown),
+    m_nextAction(Transfers::Continue)
 {
     if (!m_instance) {
         m_instance = this;
@@ -35,11 +37,16 @@ TransferModel::TransferModel() :
     m_roleNames[Transfer::ConvertToAudioRole] = "convertToAudio";
     m_roleNames[Transfer::PreferredConnectionsRole] = "preferredConnections";
     m_roleNames[Transfer::MaximumConnectionsRole] = "maximumConnections";
+    m_roleNames[Transfer::DownloadResumableRole] = "downloadResumable";
+    m_roleNames[Transfer::TransferCountRole] = "transferCount";
     m_roleNames[Transfer::IdRole] = "id";
     m_roleNames[Transfer::PackageIdRole] = "packageId";
     m_roleNames[Transfer::PackageNameRole] = "packageName";
     m_roleNames[Transfer::PackageStatusRole] = "packageStatus";
     m_roleNames[Transfer::RowNumberRole] = "rowNumber";
+#ifdef QML_USER_INTERFACE
+    m_roleNames[Transfer::ExpandedRole] = "expanded";
+#endif
 #if (QT_VERSION >= 0x040600) && (QT_VERSION < 0x050000)
     this->setRoleNames(m_roleNames);
 #endif
@@ -49,7 +56,7 @@ TransferModel::TransferModel() :
     this->connect(m_queueTimer, SIGNAL(timeout()), this, SLOT(startNextTransfers()));
     this->connect(UrlChecker::instance(), SIGNAL(urlReady(QUrl,QString,QString)), this, SLOT(addTransfer(QUrl,QString,QString)));
     this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)), this, SLOT(onMaximumConcurrentTransfersChanged(int,int)));
-    this->connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(storeTransfers()));
+    this->connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(storeAndDeleteTransfers()));
 }
 
 TransferModel::~TransferModel() {}
@@ -305,6 +312,14 @@ void TransferModel::setStatusFilter(Transfers::Status status) {
     }
 }
 
+Transfers::Action TransferModel::nextAction() const {
+    return m_nextAction;
+}
+
+void TransferModel::setNextAction(Transfers::Action action) {
+    m_nextAction = action;
+}
+
 void TransferModel::filter() {
     int pos = m_filteredTransfers.size() - 1;
 
@@ -347,7 +362,7 @@ int TransferModel::activeTransfers() const {
 
 void TransferModel::addTransfer(const QUrl &url, const QString &service, const QString &fileName) {
     Transfer *transfer = new Transfer;
-    transfer->setId(QString::number(QDateTime::currentMSecsSinceEpoch()));
+    transfer->setId(QString::number(Utils::currentMSecsSinceEpoch()));
     transfer->setDownloadPath(Settings::instance()->downloadPath() + ".incomplete/" + transfer->id());
     transfer->setFileName(fileName);
     transfer->setUrl(url);
@@ -380,8 +395,11 @@ void TransferModel::addTransfer(const QUrl &url, const QString &service, const Q
 
         emit countChanged(this->rowCount());
     }
-
+#ifdef QML_USER_INTERFACE
+    this->connect(transfer, SIGNAL(speedChanged()), this, SLOT(onTransferSpeedChanged()));
+#else
     this->connect(transfer, SIGNAL(dataChanged(int)), this, SLOT(onTransferDataChanged(int)));
+#endif
     this->connect(transfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
 
     if ((Settings::instance()->startTransfersAutomatically()) && (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers())) {
@@ -390,10 +408,6 @@ void TransferModel::addTransfer(const QUrl &url, const QString &service, const Q
     }
 }
 
-QString TransferModel::generateTransferId(QString seedFileName) const {
-    return QString("%1_%2").arg(QString::number(QDateTime::currentMSecsSinceEpoch())).arg(seedFileName.left(seedFileName.lastIndexOf('.')));
-}
-
 void TransferModel::getNextTransfers() {
     int priority = Transfers::HighPriority;
 
@@ -515,6 +529,11 @@ bool TransferModel::cancel(const QString &id) {
     return false;
 }
 
+#ifdef QML_USER_INTERFACE
+void TransferModel::onTransferSpeedChanged() {
+    emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
+}
+#else
 void TransferModel::onTransferDataChanged(int role) {
     Q_UNUSED(role)
 
@@ -534,63 +553,123 @@ void TransferModel::onTransferDataChanged(int role) {
         }
     }
 }
+#endif
 
 void TransferModel::onTransferStatusChanged(Transfers::Status status) {
-    if (status <= Transfers::Queued) {
+    switch (status) {
+    case Transfers::Queued:
+        if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
+            m_queueTimer->start();
+        }
+
+        break;
+    case Transfers::Paused:
+    case Transfers::LongWait:
+    {
         if (Transfer *transfer = qobject_cast<Transfer*>(this->sender())) {
             this->removeActiveTransfer(transfer);
-
-            if ((status == Transfers::Completed) || (status == Transfers::Cancelled)) {
-                if (transfer->parentTransfer()) {
-                    if (transfer->parentTransfer() == m_rootItem) {
-                        this->beginRemoveRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
-                    }
-                    else {
-                        this->beginRemoveRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
-                    }
-
-                    transfer->parentTransfer()->removeChildTransfer(transfer);
-
-                    this->endRemoveRows();
-                }
-
-                if (transfer->count() > 0) {
-                    if (Transfer *firstChild = transfer->removeChildTransfer(0)) {
-                        foreach (Transfer *childTransfer, transfer->childTransfers()) {
-                            firstChild->addChildTransfer(childTransfer);
-                        }
-
-                        if (transfer->parentTransfer()) {
-                            if (transfer->parentTransfer() == m_rootItem) {
-                                this->beginInsertRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
-                            }
-                            else {
-                                this->beginInsertRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
-                            }
-
-                            transfer->parentTransfer()->insertChildTransfer(transfer->rowNumber(), firstChild);
-
-                            this->endInsertRows();
-                        }
-                    }
-                }
-
-                transfer->deleteLater();
-
-                emit countChanged(this->rowCount());
-
-                if (this->rowCount() == 0) {
-                    QMetaObject::invokeMethod(Storage::instance(), "clearStoredTransfers", Qt::QueuedConnection);
-                }
-            }
-
-            emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
         }
 
         if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
             m_queueTimer->start();
         }
+
+        break;
     }
+    case Transfers::Failed:
+    {
+        if (Transfer *transfer = qobject_cast<Transfer*>(this->sender())) {
+            this->removeActiveTransfer(transfer);
+        }
+
+        switch (this->nextAction()) {
+        case Transfers::Pause:
+            if (m_activeTransfers.isEmpty()) this->pause();
+            break;
+        case Transfers::Quit:
+            if (m_activeTransfers.isEmpty()) QCoreApplication::instance()->quit();
+            return;
+        default:
+            if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
+                m_queueTimer->start();
+            }
+
+            break;
+        }
+
+        break;
+    }
+    case Transfers::Cancelled:
+    case Transfers::Completed:
+    {
+        if (Transfer *transfer = qobject_cast<Transfer*>(this->sender())) {
+            this->removeActiveTransfer(transfer);
+
+            if (transfer->parentTransfer()) {
+                if (transfer->parentTransfer() == m_rootItem) {
+                    this->beginRemoveRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
+                }
+                else {
+                    this->beginRemoveRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
+                }
+
+                transfer->parentTransfer()->removeChildTransfer(transfer);
+
+                this->endRemoveRows();
+            }
+
+            if (transfer->count() > 0) {
+                if (Transfer *firstChild = transfer->removeChildTransfer(0)) {
+                    foreach (Transfer *childTransfer, transfer->childTransfers()) {
+                        firstChild->addChildTransfer(childTransfer);
+                    }
+
+                    if (transfer->parentTransfer()) {
+                        if (transfer->parentTransfer() == m_rootItem) {
+                            this->beginInsertRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
+                        }
+                        else {
+                            this->beginInsertRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
+                        }
+
+                        transfer->parentTransfer()->insertChildTransfer(transfer->rowNumber(), firstChild);
+
+                        this->endInsertRows();
+                    }
+                }
+            }
+
+            transfer->deleteLater();
+
+            emit countChanged(this->rowCount());
+
+            if (this->rowCount() == 0) {
+                QMetaObject::invokeMethod(Storage::instance(), "clearStoredTransfers", Qt::QueuedConnection);
+            }
+
+            if (status == Transfers::Completed) {
+                switch (this->nextAction()) {
+                case Transfers::Pause:
+                    if (m_activeTransfers.isEmpty()) this->pause();
+                    break;
+                case Transfers::Quit:
+                    if (m_activeTransfers.isEmpty()) QCoreApplication::instance()->quit();
+                    return;
+                default:
+                    if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
+                        m_queueTimer->start();
+                    }
+
+                    break;
+                }
+            }
+        }
+    }
+    default:
+        break;
+    }
+
+    emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
 
     if ((!this->searchQuery().isEmpty()) && (this->statusFilter() != Transfers::Unknown)) {
         this->filter();
@@ -635,7 +714,11 @@ void TransferModel::onMaximumConcurrentTransfersChanged(int oldMaximum, int newM
 }
 
 void TransferModel::storeTransfers() {
-    QMetaObject::invokeMethod(Storage::instance(), "storeTransfers", Qt::QueuedConnection, Q_ARG(QList<Transfer*>, m_rootItem->childTransfers()));
+    Storage::instance()->storeTransfers(m_rootItem->childTransfers(), false);
+}
+
+void TransferModel::storeAndDeleteTransfers() {
+    Storage::instance()->storeTransfers(m_rootItem->childTransfers(), true);
 }
 
 void TransferModel::restoreStoredTransfers() {
@@ -645,21 +728,29 @@ void TransferModel::restoreStoredTransfers() {
 
 void TransferModel::onTransfersRestored(const QList<Transfer *> &transfers) {
     this->disconnect(Storage::instance(), SIGNAL(transfersRestored(QList<Transfer*>)), this, SLOT(onTransfersRestored(QList<Transfer*>)));
-    this->beginResetModel();
+    this->beginInsertRows(QModelIndex(), 0, transfers.size() - 1);
 
     foreach (Transfer *transfer, transfers) {
         m_rootItem->addChildTransfer(transfer);
 
+#ifdef QML_USER_INTERFACE
+        this->connect(transfer, SIGNAL(speedChanged()), this, SLOT(onTransferSpeedChanged()));
+#else
         this->connect(transfer, SIGNAL(dataChanged(int)), this, SLOT(onTransferDataChanged(int)));
+#endif
         this->connect(transfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
 
         foreach (Transfer *childTransfer, transfer->childTransfers()) {
+#ifdef QML_USER_INTERFACE
+            this->connect(childTransfer, SIGNAL(speedChanged()), this, SLOT(onTransferSpeedChanged()));
+#else
             this->connect(childTransfer, SIGNAL(dataChanged(int)), this, SLOT(onTransferDataChanged(int)));
+#endif
             this->connect(childTransfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
         }
     }
 
-    this->endResetModel();
+    this->endInsertRows();
 
     emit countChanged(this->rowCount());
 
